import json

from django.test import TestCase

from ubuntu_sso_saml.models import SAMLConfig
from ubuntu_sso_saml.tests.helpers import find_cert_path
from ubuntu_sso_saml.utils import (
    certificate_from_sp_config,
    get_config_and_link_for_resource,
    get_config_for_acs,
    get_deeplink_url,
)


class GetConfigTestCase(TestCase):

    def test_get_config_for_acs(self):
        conf = SAMLConfig.objects.create(
            acs_url='http://localhost', enabled=True)
        self.assertEqual(get_config_for_acs('http://localhost'), conf)

    def test_get_config_for_acs_when_disabled(self):
        SAMLConfig.objects.create(
            acs_url='http://localhost', enabled=False)
        self.assertIsNone(get_config_for_acs('http://localhost'))

    def test_get_config_for_acs_not_found(self):
        self.assertIsNone(get_config_for_acs('http://localhost'))

    def test_get_config_and_link_for_resource(self):
        links = {
            'foo': 'http://example.com/foo',
        }
        conf = SAMLConfig.objects.create(
            acs_url='http://localhost', links=json.dumps(links),
            enabled=True)
        sp_config, link = get_config_and_link_for_resource('foo')
        self.assertEqual(sp_config, conf)
        self.assertEqual(link, ('foo', 'http://example.com/foo'))

    def test_get_config_and_link_for_resource_when_disabled(self):
        links = {
            'foo': 'http://example.com/foo',
        }
        SAMLConfig.objects.create(
            acs_url='http://localhost', links=json.dumps(links),
            enabled=False)
        sp_config, link = get_config_and_link_for_resource('foo')
        self.assertIsNone(sp_config)
        self.assertIsNone(link)

    def test_get_config_and_link_for_resource_not_found(self):
        links = {
            'foo': 'http://example.com/foo',
        }
        SAMLConfig.objects.create(
            acs_url='http://localhost', links=json.dumps(links))
        sp_config, link = get_config_and_link_for_resource('foo')
        self.assertIsNone(sp_config)
        self.assertIsNone(link)


class GetDeeplinkUrlTestCase(TestCase):

    def test_get_deeplink_url_simple_pattern(self):
        resource_name = 'prefix/foo'
        resource_pattern = 'prefix/(?P<target>.+)$'
        url_pattern = 'http://example.com/%(target)s'

        url = get_deeplink_url(resource_name, (resource_pattern, url_pattern))
        self.assertEqual(url, 'http://example.com/foo')

    def test_get_deeplink_url_complex_pattern(self):
        resource_name = 'prefix/foo/bar/baz'
        resource_pattern = 'prefix/(?P<one>.+)/(?P<two>.+)/(?P<three>.+)$'
        url_pattern = 'http://example.com/%(one)s/%(two)s-%(three)s'

        url = get_deeplink_url(resource_name, (resource_pattern, url_pattern))
        self.assertEqual(url, 'http://example.com/foo/bar-baz')


class CertificateFromSpConfigTestCase(TestCase):

    def test_certificate_from_no_sp_config(self):
        self.assertEqual(None, certificate_from_sp_config(None))

    def test_certificate_from_sp_config_no_cert(self):
        sp_config = SAMLConfig.objects.create(
            acs_url='http://localhost')
        self.assertEqual(None, certificate_from_sp_config(sp_config))

    def test_certificate_from_sp_config_with_cert(self):
        with open(find_cert_path('certificate.pem')) as cert_file:
            cert_string = cert_file.read()
        sp_config = SAMLConfig.objects.create(
            acs_url='http://localhost', certificate=cert_string)
        # Ensure it's identical to a what we read from the file, and not
        # a djangoized unicode.
        self.assertEqual(cert_string, certificate_from_sp_config(sp_config))
